// Copyright © 2022-2025 Gerald B. Cox
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.

package mpd

import (
	"bytes"
	"encoding/json"
	"errors"
	"fmt"
	"io"
	"log/slog"
	"net/http"
	"strings"

	"codeberg.org/gbcox/beetbrainz/common"
	"codeberg.org/gbcox/beetbrainz/types"
)

type MPDRequest struct {
	Event string          `json:"Event"`
	User  string          `json:"User"`
	Item  types.MediaItem `json:"Item"`
}

func HandleRequest(w http.ResponseWriter, r *http.Request) {
	if r.Method != "POST" {
		slog.Warn("MPD webhook received unsupported method", "method", r.Method, "expected_method", "POST")
		http.Error(w, "Method Not Allowed", http.StatusMethodNotAllowed)
		return
	}

	rq, err := parseRequest(r)
	if err != nil {
		slog.Error("Failed to parse MPD webhook request", "error", err)
		http.Error(w, fmt.Sprintf("Bad Request: %s", err.Error()), http.StatusBadRequest)
		return
	}

	common.HandleRequest(rq)
	w.WriteHeader(http.StatusOK)
}

func parseRequest(r *http.Request) (*common.Request, error) {
	var mpdRequest MPDRequest
	var err error

	bodyBytes, err := io.ReadAll(r.Body)
	if err != nil {
		return nil, fmt.Errorf("failed to read request body: %w", err)
	}
	r.Body = io.NopCloser(bytes.NewBuffer(bodyBytes))

	slog.Debug("Received MPD webhook raw body", "body", string(bodyBytes))

	contentType := r.Header.Get("Content-Type")

	if strings.Contains(contentType, "application/json") {
		err = json.NewDecoder(r.Body).Decode(&mpdRequest)
		if err != nil {
			return nil, fmt.Errorf("failed to decode JSON from request body: %w", err)
		}
	} else {
		return nil, fmt.Errorf("unsupported Content-Type: %s", contentType)
	}

	slog.Debug("Parsed MPD webhook request",
		"event", mpdRequest.Event,
		"user", mpdRequest.User,
		"item_track", mpdRequest.Item.Track,
		"item_album", mpdRequest.Item.Album,
		"item_artist", mpdRequest.Item.Artist,
		"item_mbid_artist", mpdRequest.Item.MusicBrainzArtistID,
		"item_mbid_album", mpdRequest.Item.MusicBrainzAlbumID,
		"item_mbid_track", mpdRequest.Item.MusicBrainzTrackID,
	)

	if mpdRequest.Event == "" {
		return nil, errors.New("MPD webhook payload missing 'Event' field")
	}
	if mpdRequest.Item.Track == "" || mpdRequest.Item.Album == "" {
		return nil, errors.New("MPD webhook payload missing required music metadata (Track or Album)")
	}
	if mpdRequest.User == "" {
		return nil, errors.New("MPD webhook payload missing 'User' field")
	}

	rq := &common.Request{
		Event:     mpdRequest.Event,
		MediaType: "audio",
		User:      mpdRequest.User,
		Item:      &mpdRequest.Item,
		Source:    "mpd",
	}

	slog.Debug("Prepared common.Request for handling",
		"event", rq.Event,
		"media_type", rq.MediaType,
		"user", rq.User,
		"item_string_for_lookup", rq.Item.String(),
		"item_artist_for_lookup", rq.Item.Artist,
		"item_album_for_lookup", rq.Item.Album,
		"item_track_for_lookup", rq.Item.Track,
		"item_mbid_artist_for_lookup", rq.Item.MusicBrainzArtistID,
		"item_mbid_album_for_lookup", rq.Item.MusicBrainzAlbumID,
		"item_mbid_track_for_lookup", rq.Item.MusicBrainzTrackID,
	)

	return rq, nil
}
